/**
 * $Id: MBeanHelper.java,v 1.1 2009/02/20 21:34:03 oboehm Exp $
 *
 * Copyright (c) 2008 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * (c)reated 19.02.2009 by oliver (ob@oasd.de)
 */
package patterntesting.runtime.jmx;

import java.lang.management.ManagementFactory;

import javax.management.*;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.impl.LogFactoryImpl;

/**
 * @author <a href="boehm@javatux.de">oliver</a>
 * @since 19.02.2009
 * @version $Revision: 1.1 $
 */
public class MBeanHelper {
    
    private static final Log log = LogFactoryImpl.getLog(MBeanHelper.class);
    private static MBeanServer server = ManagementFactory.getPlatformMBeanServer();
    
    public static String getMBeanName(Object mbean) {
        return getMBeanName(mbean.getClass());
    }
    
    /**
     * Converts the class name into "p.a.c.k.a.g.e:type=Classname".
     *
     * @param cl e.g. my.hello.World
     * @return e.g. "my.hello:type=World"
     */
    public static String getMBeanName(Class<?> cl) {
        return cl.getPackage().getName() + ":type=" + cl.getSimpleName();
    }
    
    public static void registerMBean(Object mbean) throws JMException {
        String mbeanName = getMBeanName(mbean);
        registerMBean(mbeanName, mbean);
    }
    
    public static synchronized void registerMBean(String mbeanName, Object mbean) {
        try {
            ObjectName name = new ObjectName(mbeanName);
            registerMBean(name, mbean);
        } catch (MalformedObjectNameException e) {
            log.info("can't register <" + mbean + "> as MBean", e);
        }
    }
    
    public static synchronized void registerMBean(ObjectName name, Object mbean) {
        try {
            log.trace("registering " + name + "...");
            server.registerMBean(mbean, name);
            log.debug(name + " successful registered as MBean");
        } catch (Exception e) {
            log.info("can't register <" + mbean + "> as MBean", e);
        }
    }
    
    public static synchronized void unregisterMBean(String mbeanName) {
        try {
            ObjectName name = new ObjectName(mbeanName);
            server.unregisterMBean(name);
            log.debug("MBean " + name + " successful unregistered");
        } catch (Exception e) {
            log.info("can't unregister " + mbeanName, e);
        }
    }
    
    public static boolean isRegistered(String mbeanName) {
        try {
            ObjectName name = new ObjectName(mbeanName);
            ObjectInstance mbean = server.getObjectInstance(name);
            return (mbean != null);
        } catch (Exception e) {
            log.trace(mbeanName + " not found (" + e + ")");
            return false;
        }
    }

}

/**
 * $Log: MBeanHelper.java,v $
 * Revision 1.1  2009/02/20 21:34:03  oboehm
 * MBeanRegistry with default implementation added
 *
 * $Source: /cvsroot/patterntesting/PatternTesting08/patterntesting-rt/src/main/java/patterntesting/runtime/jmx/MBeanHelper.java,v $
 */
