/**
 * $Id: ReflectionHelper.java,v 1.1 2009/03/09 21:18:54 oboehm Exp $
 *
 * Copyright (c) 2009 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * (c)reated 09.03.2009 by oliver (ob@aosd.de)
 */
package patterntesting.runtime.util;

import java.lang.reflect.Field;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.impl.LogFactoryImpl;

/**
 * This class is a helper class to access some fields via reflection.
 * Normally you should avoid reflection. Handle with care if you use it.
 * 
 * @author <a href="boehm@javatux.de">oliver</a>
 * @since 09.03.2009
 * @version $Revision: 1.1 $
 */
public class ReflectionHelper {
    
    private static final Log log = LogFactoryImpl.getLog(ReflectionHelper.class);

    public static Field getField(Class<?> cl, String name) {
        try {
            return cl.getDeclaredField(name);
        } catch (SecurityException e) {
            log.warn("can't access field " + cl + "#" + name, e);
            return null;
        } catch (NoSuchFieldException e) {
            Class<?> superclass = cl.getSuperclass();
            if (superclass == null) {
                log.debug("field " + name + " not found in " + cl);
                return null;
            } else {
                return ReflectionHelper.getField(superclass, name);
            }
        }
    }
    
    /**
     * If it can it returns the value of the given field.
     * If not it returns null.
     * 
     * @param obj
     * @param name
     * @return
     */
    public static Object getFieldValue(Object obj, String name) {
        Field field = ReflectionHelper.getField(obj.getClass(), name);
        if (field == null) {
            return null;
        }
        field.setAccessible(true);
        try {
            return field.get(obj);
        } catch (Exception e) {
            log.warn("can't access " + field, e);
            return null;
        }
    }

}

/**
 * $Log: ReflectionHelper.java,v $
 * Revision 1.1  2009/03/09 21:18:54  oboehm
 * refactored out some parts to ReflectionHelper
 *
 * $Source: /cvsroot/patterntesting/PatternTesting08/patterntesting-rt/src/main/java/patterntesting/runtime/util/ReflectionHelper.java,v $
 */
