/**
 * $Id: ThreadUtil.java,v 1.2 2009/01/09 21:25:17 oboehm Exp $
 *
 * Copyright (c) 2008 by Oliver Boehm
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express orimplied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * (c)reated 30.09.2008 by oliver (ob@oasd.de)
 */
package patterntesting.runtime.util;

import java.util.concurrent.TimeUnit;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.impl.LogFactoryImpl;

/**
 * @author <a href="boehm@javatux.de">oliver</a>
 * @since 30.09.2008
 * @version $Revision: 1.2 $
 */
public class ThreadUtil {
	
	private static Log log = LogFactoryImpl.getLog(ThreadUtil.class);
	private static final long timeInMillis;
	
	static {
		timeInMillis = calibrateMillis();
		log.debug("timeInMillis calibrated (" + timeInMillis + "ms)");
	}
	
	public static void sleep() {
		sleep(timeInMillis);
	}
	
	public static void sleep(long millis) {
		try {
			Thread.sleep(millis);
		} catch (InterruptedException ignored) {
			log.info(ignored + " ignored");
		}
	}
	
	public static void sleep(long millis, int nanos) {
		try {
			Thread.sleep(millis, nanos);
		} catch (InterruptedException ignored) {
			log.info(ignored + " ignored");
		}
	}

	/**
	 * To be able to support Java 5 the constants MINUTES, HOURS and DAYS
	 * are not used.
	 * @param time
	 * @param unit
	 */
	public static void sleep(int time, TimeUnit unit) {
		switch (unit) {
		case NANOSECONDS:
			sleep(0L, time);
			break;
		case MICROSECONDS:
			sleep(0L, time * 1000);
			break;
		case MILLISECONDS:
			sleep(time);
			break;
		case SECONDS:
			sleep(time * 1000L);
			break;
//		case MINUTES:
//			sleep(time * 1000L * 60);
//			break;
//		case HOURS:
//			sleep(time * 1000L * 60 * 60);
//			break;
//		case DAYS:
//			sleep(time * 1000L * 60 * 60 * 24);
//			break;
		default:
			log.warn("unknown TimeUnit " + unit + " interpreting it as MINUTES");
			sleep(time * 1000L * 60);
			break;
		}
	}

	/**
	 * Determines the timer resolution for the sleep() method.
	 * On windows this is 15 ms, on Linux and MacOS 1 ms.
	 */
	private static long calibrateMillis() {
		long t0 = nextTimeMillis();
		long t1 = nextTimeMillis();
		return t1 - t0;
	}
	
	/**
	 * Go to the next timer tick
	 * @return the next timestamp
	 */
	private static long nextTimeMillis() {
		long t0 = System.currentTimeMillis();
		while(t0 == System.currentTimeMillis()) {
			// do nothing
		}
		return System.currentTimeMillis();
	}
	
	public static long getTimerResolutionInMillis() {
		return timeInMillis;
	}

}

/**
 * $Log: ThreadUtil.java,v $
 * Revision 1.2  2009/01/09 21:25:17  oboehm
 * Java6 features removed
 *
 * Revision 1.1  2008/12/01 21:38:12  oboehm
 * some reorganisation
 * warning is now printed for @Synchronized methods
 * (without @Synchronized class)
 *
 * Revision 1.2  2008/11/19 22:01:15  oboehm
 * sleep(int, TimeUnit) added
 *
 * Revision 1.1  2008/11/06 22:12:43  oboehm
 * begin of split in runtime components and others
 *
 * Revision 1.2  2008/10/14 20:13:08  oboehm
 * ThreadUtil calibrates itself for sleep() method
 *
 * Revision 1.1  2008/10/02 20:36:32  oboehm
 * methods in @TreadSafe classes will be synchronized
 *
 * $Source: /cvsroot/patterntesting/PatternTesting08/patterntesting-rt/src/main/java/patterntesting/runtime/util/ThreadUtil.java,v $
 */
